from abc import ABC, abstractmethod


class DatabasePluginBase(ABC):
    """
    The DatabasePluginBase is the base class of all database backend plugins supported by cve search. The default
    backend plugin is mongodb. If you wish to use a different backend you could provide your own plugin. Plugins should
    be placed within the lib.DatabasePlugins folder. Plugins should be a class derived from this class and the class
    name should end with 'Plugin' for it to be automatically picked up by the DatabasePluginLoader and presented to cve
    search. The name of the plugin should be defined in the 'PluginName' variable of the configuration.ini file and
    should be the same as the name of the file (minus the '.py') in which the plugin class resides.

    ******************************************************************************************************************
            FOR Now only a small part of the API makes use of this plugin based database access all other
                            endpoints and functionalities do not use these plugins yet
    ******************************************************************************************************************
    """

    def __init__(self):
        pass

    @abstractmethod
    def query_docs(self, **kwargs):
        """
        Method used to perform a extended query against the database in order to fetch data.

        This method should accept:

        :param retrieve: Collection/table name where to retrieve the documents from
        :type retrieve: str
        :param dict_filter: A column filter based on a python dictionary
        :type dict_filter: dict
        :param sort: Field to sort on or list of tuples of field and direction; e.g. (for mongodb)
                    [("field", pymongo.DESCENDING), ("field2", pymongo.ASCENDING)]
        :type sort: str or list
        :param limit: Can be used to limit the amount of returned results
        :type limit: int
        :param skip: Can be used to skip the first x records of the returned results
        :type skip: int
        :param query_filter: Python Dictionary to include fields to return in the query results
                             (e.g.: {field1: 1, field2: 1})
        :type query_filter: dict
        :param sort_dir: A general sorting direction; e.g. DESC for DESCENDING, ASC for ASCENDING
        :type sort_dir: str

        This method should return:

        :return: A list of dictionaries whith the data saved in the backend. Each dictionary represents a single
                 row/document and the dictionary keys represent the fields/columns in the row/document
        :rtype: list

        """
        raise NotImplementedError

    @abstractmethod
    def fetch_one(self, **kwargs):
        """
        The fetch one method returns a single record based on the provided parameters.

        This method should accept:

        :param retrieve: Collection name where to retrieve the documents from
        :type retrieve: str
        :param dict_filter: A column filter based on a python dictionary
        :type dict_filter: dict
        :param query_filter: Dict with fields to exclude or include; e.g. {"hosts": 0} or {"hosts": 1}
        :type query_filter: dict

        This method should return:

        :return: Dictionary with items from database or None
        :rtype: dict or None
        """
        raise NotImplementedError

    @abstractmethod
    def create_schema(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def count(self, **kwargs):
        """
        Method for retrieving a document count

        This method should accept:

        :param retrieve: Collection/table to retrieve the count from
        :type retrieve: str
        :param dict_filter: A column filter based on a python dictionary
        :type dict_filter: dict

        This method should return:

        :return: Count
        :rtype: int
        """
        raise NotImplementedError

    @abstractmethod
    def datatables_data_columns_ordering(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def datatables_data_filter(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def datatables_data_order(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def addUser(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def changePassword(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def verifyUser(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def deleteUser(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def setAdmin(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def setLocalOnly(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def isMasterAccount(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def userExists(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def isSingleMaster(self, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def getUser(self, **kwargs):
        raise NotImplementedError



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Script to check and ensure that the recommended indexes are created.
#
# NOTE: Legacy tool. Indexes are created automatically during database
# updates, so running this script is generally unnecessary. Can still
# be used to refresh indexes without performing a full update.
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2014       psychedelys
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com
# Imports
import logging
import os
import sys

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.Config import Configuration
from CveXplore.core.database_indexer.db_indexer import DatabaseIndexer
from lib.LogHandler import UpdateHandler

logging.setLoggerClass(UpdateHandler)

# pass configuration to CveXplore
Configuration.setCveXploreEnv()


class DatabaseWrapper:
    """
    Adapt a raw pymongo Database object for CveXplore's DatabaseIndexer that
    expect a '.dbclient' attribute exposing the underlying MongoDB database.
    Unfortunately, the 'Configuration.getMongoConnection()' already returns a
    'pymongo.database.Database' instance, which does not define '.dbclient'.
    This wrapper resolves that mismatch.
    """

    def __init__(self, db):
        self.dbclient = db


if __name__ == "__main__":
    db = Configuration.getMongoConnection()
    di = DatabaseIndexer(DatabaseWrapper(db))
    di.create_indexes()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Tool to dump in JSON the database along with the associated ranking
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com
import argparse
import json
import os
import sys

from bson import json_util

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.CVEs import CveHandler
from lib.DatabaseLayer import getCVEIDs

argParser = argparse.ArgumentParser(description="Dump database in JSON format")
argParser.add_argument(
    "-r", default=False, action="store_true", help="Include ranking value"
)
argParser.add_argument(
    "-v", default=False, action="store_true", help="Include via4 map"
)
argParser.add_argument(
    "-c", default=False, action="store_true", help="Include CAPEC information"
)
argParser.add_argument(
    "-l",
    default=False,
    type=int,
    help="Limit output to n elements (default: unlimited)",
)
args = argParser.parse_args()

rankinglookup = args.r
via4lookup = args.v
capeclookup = args.c

cves = CveHandler(
    rankinglookup=rankinglookup, via4lookup=via4lookup, capeclookup=capeclookup
)

for cveid in getCVEIDs(limit=args.l):
    item = cves.getcve(cveid=cveid)
    if "cvss" in item:
        if type(item["cvss"]) == str:
            item["cvss"] = float(item["cvss"])
    date_fields = ["cvssTime", "modified", "published"]
    for field in date_fields:
        if field in item:
            item[field] = str(item[field])
    print(json.dumps(item, sort_keys=True, default=json_util.default))



import pytest

from test.runners.cli_test_runner import CLITestRunner


@pytest.fixture
def runner():
    return CLITestRunner()


def test_dump_last_atom(runner):
    result = runner.runcommand("bin/dump_last.py -f atom -l 9")

    test_results = result.stdout.replace("\n", "")

    assert result.returncode == 0
    assert test_results[39:81] == '<feed xmlns="http://www.w3.org/2005/Atom">'
    assert test_results[99:109] == "Last 9 CVE"


def test_dump_last_rss1(runner):
    result = runner.runcommand("bin/dump_last.py -f rss1 -l 9")

    assert result.returncode == 0


def test_dump_last_rss2(runner):
    result = runner.runcommand("bin/dump_last.py -f rss2 -l 9")

    assert result.returncode == 0
    assert (
        result.stdout.replace("\n", "")[0:58]
        == '<?xml version="1.0" encoding="UTF-8" ?><rss version="2.0">'
    )


def test_dump_last_html(runner):
    result = runner.runcommand("bin/dump_last.py -f html -l 9")

    assert result.returncode == 0
    assert result.stdout[:6] == "<html>"


def test_dump_last_with_capec_and_cveranking(runner):
    result = runner.runcommand("bin/dump_last.py -f atom -c -r -l 9")

    assert result.returncode == 0



import re
from html import escape
from flask import request
from lib.Config import Configuration
from web.helpers.common import timestringTOdatetime

config = Configuration()

defaultFilters = {
    "timeSelect": "all",
    "startDate": "",
    "endDate": "",
    "timeTypeSelect": "modified",
    "cvssSelect": "all",
    "cvss": "0",
    "rejectedSelect": "hide",
}

config_args = {
    "pageLength": config.getPageLength(),
    "minimal": True,
}


def generate_minimal_query(f):
    query = []
    # retrieving lists
    if f["rejectedSelect"] == "hide":
        query.append(
            {
                "summary": re.compile(
                    r"^(?!\*\* REJECT \*\*\s+DO NOT USE THIS CANDIDATE NUMBER.*)"
                )
            }
        )

    # cvss score logic
    if "cvssVersion" in f:
        if f["cvssVersion"] == "V4":
            cvss_filter_field = "cvss4"
        elif f["cvssVersion"] == "V3":
            cvss_filter_field = "cvss3"
        else:
            cvss_filter_field = "cvss"
    else:
        cvss_filter_field = "cvss3"

    if f["cvssSelect"] == "above":
        query.append({cvss_filter_field: {"$gt": float(f["cvss"])}})
    elif f["cvssSelect"] == "equals":
        query.append({cvss_filter_field: float(f["cvss"])})
    elif f["cvssSelect"] == "below":
        query.append({cvss_filter_field: {"$lt": float(f["cvss"])}})

    # date logic
    if f["timeSelect"] != "all":
        if f["startDate"]:
            startDate = timestringTOdatetime(f["startDate"])
        if f["endDate"]:
            endDate = timestringTOdatetime(f["endDate"])

        if f["timeSelect"] == "from":
            query.append({f["timeTypeSelect"]: {"$gt": startDate}})
        elif f["timeSelect"] == "until":
            query.append({f["timeTypeSelect"]: {"$lt": endDate}})
        elif f["timeSelect"] == "between":
            query.append({f["timeTypeSelect"]: {"$gt": startDate, "$lt": endDate}})
        elif f["timeSelect"] == "outside":
            query.append(
                {
                    "$or": [
                        {f["timeTypeSelect"]: {"$lt": startDate}},
                        {f["timeTypeSelect"]: {"$gt": endDate}},
                    ]
                }
            )
    return query


def filter_logic(filters, skip, limit=None):
    from lib.DatabaseLayer import getCVEs

    query = generate_minimal_query(filters)
    limit = limit if limit else config_args["pageLength"]
    cve = getCVEs(limit=limit, skip=skip, query=query)
    return cve


def getFilterSettingsFromPost(r):
    from lib.DatabaseLayer import getCVEs

    filters = dict(request.form)
    errors = False
    # retrieving data
    try:
        cve = filter_logic(filters, r)
    except Exception:
        cve = getCVEs(limit=config_args["pageLength"], skip=r)
        errors = True
    return {"filters": filters, "cve": cve, "errors": errors}


def filterUpdateField(data):
    if not data:
        return data
    returnvalue = []
    for line in data.split("\n"):
        if (
            not line.startswith("[+]Success to create index")
            and not line == "Not modified"
            and not line.startswith("Starting")
        ):
            returnvalue.append(line)
    return "\n".join(returnvalue)


def validateFilter(filter_params):
    # CVSS must be a number betreen 0 and 10.
    try:
        cvss = float(filter_params["cvss"])
        if cvss > 10:
            return False
        if cvss < 0:
            return False
    except ValueError:
        return False

    # Require valid options for selections.
    if filter_params["cvssSelect"] not in ["all", "above", "equals", "below"]:
        return False
    if filter_params["cvssVersion"] not in ["V2", "V3", "V4"]:
        return False
    if filter_params["rejectedSelect"] not in ["hide", "show"]:
        return False
    if filter_params["timeSelect"] not in [
        "all",
        "from",
        "until",
        "between",
        "outside",
    ]:
        return False
    if filter_params["timeTypeSelect"] not in [
        "modified",
        "published",
        "lastModified",
    ]:
        return False

    # Validate dates: required values and reasonable ranges.
    if filter_params["timeSelect"] in ["from", "between", "outside"]:
        startDate = timestringTOdatetime(filter_params["startDate"])
        if not startDate:
            return False
    if filter_params["timeSelect"] in ["until", "between", "outside"]:
        endDate = timestringTOdatetime(filter_params["endDate"])
        if not endDate:
            return False
    if filter_params["timeSelect"] in ["between", "outside"]:
        if startDate > endDate:
            return False

    # None of the tests failed.
    return True


def SanitizeUserInput(filter_params):
    """
    Method to sanitize user input

    :param filter_params: Dictionary with filter params
    :type filter_params: dict
    :return: Sanitized filter params
    :rtype: dict
    """

    for each in filter_params:
        filter_params[each] = escape(filter_params[each])

    return filter_params


def adminInfo(output=None):
    from lib.DatabaseLayer import getDBStats

    return {"stats": getDBStats(True), "updateOutput": filterUpdateField(output)}


def parse_headers(headers):
    ret_dict = {}

    for key, val in headers.items():
        ret_dict[key] = val

    return ret_dict
